@@ -1,11 +1,11 @@
 ;;; cg.el --- Major mode for editing Constraint Grammar files  -*- lexical-binding: t; coding: utf-8 -*-
 
-;; Copyright (C) 2010-2022 Kevin Brubeck Unhammer
+;; Copyright (C) 2010-2024 Kevin Brubeck Unhammer
 
 ;; Author: Kevin Brubeck Unhammer <unhammer@fsfe.org>
 ;; Version: 0.4.0
 ;; Package-Requires: ((emacs "26.1"))
-;; Url: https://visl.sdu.dk/constraint_grammar.html
+;; Url: https://edu.visl.dk/constraint_grammar.html
 ;; Keywords: languages
 
 ;; This file is not part of GNU Emacs.
@@ -57,7 +57,7 @@
 ;; - use something like prolog-clause-start to define M-a/e etc.
 ;; - indentation function (based on prolog again?)
 ;; - the rest of the keywords
-;; - https://visl.sdu.dk/cg3/single/#regex-icase
+;; - https://edu.visl.dk/cg3/single/#regex-icase
 ;; - keyword tab-completion
 ;; - `font-lock-syntactic-keywords' is obsolete since 24.1
 ;; - show definition of set/list-at-point in modeline
@@ -155,16 +155,20 @@ Don't change without re-evaluating `cg-kw-re' (or all of cg.el).")
     "UNPROTECT"
     "IFF"
     "ADDCOHORT" "REMCOHORT"
+    "MERGECOHORTS"
+    "COPYCOHORT"
+    "SPLITCOHORT"
     "COPY"
     "MOVE" "SWITCH"
     "EXTERNAL" "DELIMIT"
     "MAP"    "ADD"
     "UNMAP"
     "SELECT" "REMOVE"
-    "SETPARENT"    "SETCHILD"
+    "SETPARENT"    "SETCHILD"     "REMPARENT"    "SWITCHPARENT"
     "ADDRELATION"  "REMRELATION"  "SETRELATION"
     "ADDRELATIONS" "REMRELATIONS" "SETRELATIONS"
     "SETVARIABLE"  "REMVARIABLE"
+    "CMDARGS" "CMDARGS-OVERRIDE" "OPTIONS"
     "APPEND")
   "Rule-starter keywords for indentation, highlighting etc.
 Don't change without re-evaluating `cg-kw-re' (or all of cg.el)." )
@@ -200,9 +204,16 @@ Don't change without re-evaluating `cg-kw-re' (or all of cg.el)." )
 			     "UNMAPLAST"
 			     "REVERSE"
 			     "SUB"
-			     "OUTPUT")
+				 "OUTPUT"
+				 "CAPTURE_UNIF"
+				 "REPEAT"
+				 "IGNORED"
+				 "LOOKIGNORED"
+				 "NOMAPPED"
+				 "NOPARENT"
+				 "DETACH")
   "Rule flags used for highlighting.
-from https://visl.sdu.dk/svn/visl/tools/vislcg3/trunk/src/Strings.cpp
+from https://edu.visl.dk/svn/visl/tools/vislcg3/trunk/src/Strings.cpp
 Don't change without re-evaluating the file.")
 (defconst cg-kw-context-flags '("NOT"
 				"NEGATE"
@@ -212,6 +223,7 @@ Don't change without re-evaluating the file.")
 				"CBARRIER"
 				"OR"
 				"TARGET"
+				"EXCEPT"
 				"IF"
 				"AFTER"
 				"BEFORE"
@@ -466,12 +478,14 @@ CG-mode provides the following specific keyboard key bindings:
     (font-lock-set-defaults)
     (font-lock-ensure))
   (add-hook 'after-change-functions #'cg-after-change nil 'buffer-local)
+  ;; TODO: Use new track-changes library for ↑ in Emacs >= 30.1, more performant than after-change-functions
   (let* ((buf (current-buffer))
          (hl-timer (run-with-idle-timer 1 'repeat 'cg-output-hl buf)))
     (add-hook 'kill-buffer-hook
               (lambda () (cancel-timer hl-timer))
               nil
-              'local)))
+              'local))
+  (cg-imenu-setup))
 
 
 (defconst cg-font-lock-syntactic-keywords
@@ -659,7 +673,7 @@ select the whole string \"SELECT:1022:rulename\")."
                           (yank)
                           (buffer-substring-no-properties (point-min)(point-max))))))
     (if (string-match
-         "\\(\\(select\\|iff\\|remove\\|map\\|addcohort\\|remcohort\\|switch\\|copy\\|add\\|substitute\\):\\)?\\([0-9]+\\)"
+         "\\(\\(select\\|iff\\|remove\\|map\\|addcohort\\|remcohort\\|switch\\|copy\\|add\\|substitute\\|mergecohorts\\|copycohort\\|splitcohort\\):\\)?\\([0-9]+\\)"
          rule)
         (progn (goto-char (point-min))
 	       (forward-line (1- (string-to-number (match-string 3 rule))))
@@ -953,6 +967,19 @@ you want to keep analyses hidden most of the time.")
 
 (defvar cg--output-unhide-history nil)
 
+
+
+;;; Imenu
+
+(defun cg-imenu-setup ()
+  "Set up `imenu-generic-expression' for running `imenu'."
+  (setq imenu-generic-expression
+        '((nil "^#*\\s *\\(SECTION\\|CONSTRAINTS\\).*" 0)
+          (nil "^#*\\s *DELIMITERS.*" 0)
+          (nil "^#*\\s *MAPPINGS.*" 0)
+          (nil "^#*\\s *\\(BEFORE\\|AFTER\\)-SECTIONS.*" 0))))
+
+
 
 
 (define-compilation-mode cg-output-mode "CG-out"
@@ -1174,7 +1201,10 @@ buffer to cg-output-buffer on check."
   (when string
     (process-send-string proc string))
   (process-send-string proc "\n")
-  (process-send-eof proc))
+  (process-send-eof proc)
+  ;; two eof's needed for some pre-pipes, not sure why:
+  (when (process-live-p proc)
+    (process-send-eof proc)))
 
 (defun cg-check ()
   "Run vislcg3 --trace on the buffer with your example inputs.
