@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "ContextualTest.hpp"
@@ -32,6 +30,9 @@ bool ContextualTest::operator==(const ContextualTest& other) const {
 	if (pos != other.pos) {
 		return false;
 	}
+	if (jump_pos != other.jump_pos) {
+		return false;
+	}
 	if (target != other.target) {
 		return false;
 	}
@@ -70,6 +71,7 @@ uint32_t ContextualTest::rehash() {
 	}
 
 	hash = hash_value(pos);
+	hash = hash_value(hash, jump_pos);
 	hash = hash_value(hash, target);
 	hash = hash_value(hash, barrier);
 	hash = hash_value(hash, cbarrier);
@@ -97,21 +99,6 @@ uint32_t ContextualTest::rehash() {
 	return hash;
 }
 
-void ContextualTest::resetStatistics() {
-	num_fail = 0;
-	num_match = 0;
-	total_time = 0;
-	if (tmpl) {
-		tmpl->resetStatistics();
-	}
-	for (auto idts : ors) {
-		idts->resetStatistics();
-	}
-	if (linked) {
-		linked->resetStatistics();
-	}
-}
-
 void ContextualTest::markUsed(Grammar& grammar) {
 	if (is_used) {
 		return;
