@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2023, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -53,6 +53,9 @@ public:
 	uint32_t verbosity_level = 0;
 	mutable double total_time = 0;
 
+	std::string cmdargs;
+	std::string cmdargs_override;
+
 	std::vector<Tag*> single_tags_list;
 	Taguint32HashMap single_tags;
 
