@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2023, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -92,6 +92,7 @@ struct Rule_Context {
 	unif_sets_t* unif_sets = nullptr;
 	uint8_t regexgrp_ct = 0;
 	regexgrps_t* regexgrps = nullptr;
+	bool is_with = false;
 };
 
 typedef std::function<void(void)> RuleCallback;
@@ -123,6 +124,7 @@ public:
 	bool split_mappings = false;
 	bool pipe_deleted = false;
 	bool add_spacing = true;
+	bool print_ids = false;
 
 	bool dep_has_spanned = false;
 	uint32_t dep_delimit = 0;
@@ -137,6 +139,7 @@ public:
 	uint32Vector sections;
 	uint32IntervalVector valid_rules;
 	uint32IntervalVector trace_rules;
+	uint32IntervalVector debug_rules;
 	uint32FlatHashMap variables;
 	uint32_t verbosity_level = 0;
 	uint32_t debug_level = 0;
@@ -249,6 +252,8 @@ protected:
 	Cohort* merge_with = nullptr;
 	Rule* current_rule = nullptr;
 	std::vector<Rule_Context> context_stack;
+	std::vector<CohortSet*> cohortsets;
+	std::vector<CohortSet::const_iterator*> rocits;
 
 	ReadingSpec get_attach_to();
 	Cohort* get_mark();
@@ -277,10 +282,10 @@ protected:
 	scoped_stack<unif_sets_t> ss_usets;
 	scoped_stack<uint32SortedVector> ss_u32sv;
 
-	uint32FlatHashSet index_regexp_yes;
-	uint32FlatHashSet index_regexp_no;
-	uint32FlatHashSet index_icase_yes;
-	uint32FlatHashSet index_icase_no;
+	uint64FlatHashSet index_regexp_yes;
+	uint64FlatHashSet index_regexp_no;
+	uint64FlatHashSet index_icase_yes;
+	uint64FlatHashSet index_icase_no;
 	std::vector<uint32FlatHashSet> index_readingSet_yes;
 	std::vector<uint32FlatHashSet> index_readingSet_no;
 	uint32FlatHashSet index_ruleCohort_no;
@@ -356,6 +361,34 @@ protected:
 	std::deque<Reading> subs_any;
 	Reading* get_sub_reading(Reading* tr, int sub_reading);
 
+	void printDebugRule(const Rule& rule, bool target = true, bool cntx = true) {
+		static std::stringstream buf;
+
+		bool ttrace = false;
+		swapper<bool> _st(true, trace, ttrace);
+
+		// Whole context, both before and after current window
+		buf.str("");
+		buf.clear();
+
+		buf << "# ===== BEGIN RULE " << rule.line << (target ? " TARGET-MATCH" : " TARGET-FAIL") << (cntx ? " CONTEXT-MATCH" : " CONTEXT-FAIL") << " =====\n";
+
+		buf << "# PREVIOUS WINDOWS\n";
+		for (auto s : gWindow->previous) {
+			printSingleWindow(s, buf, true);
+		}
+		buf << "# CURRENT WINDOW\n";
+		printSingleWindow(gWindow->current, buf, true);
+		buf << "# NEXT WINDOWS\n";
+		for (auto s : gWindow->next) {
+			printSingleWindow(s, buf, true);
+		}
+
+		buf << "# ===== END RULE " << rule.line << " =====\n";
+
+		u_fprintf(ux_stderr, "%s", buf.str().c_str());
+	}
+
 	template<typename T>
 	void addProfilingExample(T& item) {
 		auto& buf = profiler->buf;
