@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "GrammarApplicator.hpp"
@@ -27,13 +25,14 @@
 #include "SingleWindow.hpp"
 #include "Reading.hpp"
 #include "ContextualTest.hpp"
+#include "MathParser.hpp"
 
 namespace CG3 {
 
 /**
  * Tests whether one set is a subset of another set, specialized for TagSet.
  *
- * In the https://visl.sdu.dk/cg3_performance.html test data, this function is executed 1516098 times,
+ * In the https://edu.visl.dk/cg3_performance.html test data, this function is executed 1516098 times,
  * of which 23222 (1.5%) return true.
  *
  * @param[in] a The tags from the set
@@ -59,7 +58,7 @@ inline bool TagSet_SubsetOf_TSet(const TagSortedVector& a, const T& b) {
 }
 
 template<typename RXGS, typename Tag>
-inline void captureRegex(int32_t gc, RXGS& regexgrps, Tag& tag) {
+inline void captureRegex(int32_t gc, uint8_t& regexgrp_ct, RXGS* regexgrps, Tag& tag) {
 	constexpr auto BUFSIZE = 1024;
 	UErrorCode status = U_ZERO_ERROR;
 	UChar _tmp[BUFSIZE];
@@ -74,11 +73,11 @@ inline void captureRegex(int32_t gc, RXGS& regexgrps, Tag& tag) {
 			tmp = &_stmp[0];
 			uregex_group(tag.regexp, i, tmp, len + 1, &status);
 		}
-		regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
-		UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
+		regexgrps->resize(std::max(static_cast<size_t>(regexgrp_ct) + 1, regexgrps->size()));
+		UnicodeString& ucstr = (*regexgrps)[regexgrp_ct];
 		ucstr.remove();
 		ucstr.append(tmp, len);
-		++regexgrps.first;
+		++regexgrp_ct;
 	}
 }
 
@@ -92,11 +91,11 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 	UErrorCode status = U_ZERO_ERROR;
 	int32_t gc = uregex_groupCount(tag.regexp, &status);
 	uint32_t match = 0;
-	uint32_t ih = hash_value(tag.hash, test);
-	if (!bypass_index && index_matches(index_regexp_no, ih)) {
+	auto ih = (UI64(tag.hash) << 32) | test;
+	if (!bypass_index && index_regexp_no.contains(ih)) {
 		match = 0;
 	}
-	else if (!bypass_index && gc == 0 && index_matches(index_regexp_yes, ih)) {
+	else if (!bypass_index && gc == 0 && index_regexp_yes.contains(ih)) {
 		match = test;
 	}
 	else {
@@ -115,8 +114,8 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 			CG3Quit(1);
 		}
 		if (match) {
-			if (gc > 0 && regexgrps.second != 0) {
-				captureRegex(gc, regexgrps, tag);
+			if (gc > 0 && !context_stack.empty() && context_stack.back().regexgrps != 0) {
+				captureRegex(gc, context_stack.back().regexgrp_ct, context_stack.back().regexgrps, tag);
 			}
 			else {
 				index_regexp_yes.insert(ih);
@@ -131,11 +130,11 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 
 uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, bool bypass_index) {
 	uint32_t match = 0;
-	uint32_t ih = hash_value(tag.hash, test);
-	if (!bypass_index && index_matches(index_icase_no, ih)) {
+	auto ih = (UI64(tag.hash) << 32) | test;
+	if (!bypass_index && index_icase_no.contains(ih)) {
 		match = 0;
 	}
-	else if (!bypass_index && index_matches(index_icase_yes, ih)) {
+	else if (!bypass_index && index_icase_yes.contains(ih)) {
 		match = test;
 	}
 	else {
@@ -158,11 +157,11 @@ uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Ta
 	UErrorCode status = U_ZERO_ERROR;
 	int32_t gc = uregex_groupCount(tag.regexp, &status);
 	uint32_t match = 0;
-	uint32_t ih = hash_value(reading.tags_string_hash, tag.hash);
-	if (!bypass_index && index_matches(index_regexp_no, ih)) {
+	auto ih = (UI64(reading.tags_string_hash) << 32) | tag.hash;
+	if (!bypass_index && index_regexp_no.contains(ih)) {
 		match = 0;
 	}
-	else if (!bypass_index && gc == 0 && index_matches(index_regexp_yes, ih)) {
+	else if (!bypass_index && gc == 0 && index_regexp_yes.contains(ih)) {
 		match = reading.tags_string_hash;
 	}
 	else {
@@ -181,8 +180,8 @@ uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Ta
 		}
 		if (match) {
 			// ToDo: Allow regex captures from dependency target contexts without any captures in normal target contexts
-			if (gc > 0 && regexgrps.second != 0) {
-				captureRegex(gc, regexgrps, tag);
+			if (gc > 0 && !context_stack.empty() && context_stack.back().regexgrps != 0) {
+				captureRegex(gc, context_stack.back().regexgrp_ct, context_stack.back().regexgrps, tag);
 			}
 			else {
 				index_regexp_yes.insert(ih);
@@ -223,7 +222,7 @@ uint32_t GrammarApplicator::doesRegexpMatchReading(const Reading& reading, const
 /**
  * Tests whether a given reading matches a given tag.
  *
- * In the https://visl.sdu.dk/cg3_performance.html test data, this function is executed 1058428 times,
+ * In the https://edu.visl.dk/cg3_performance.html test data, this function is executed 1058428 times,
  * of which 827259 are treated as raw tags.
  *
  * @param[in] reading The reading to test
@@ -276,8 +275,8 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 			}
 			if (match) {
 				int32_t gc = uregex_groupCount(tag.regexp, &status);
-				if (gc > 0 && regexgrps.second != 0) {
-					captureRegex(gc, regexgrps, tag);
+				if (gc > 0 && !context_stack.empty() && context_stack.back().regexgrps != 0) {
+					captureRegex(gc, context_stack.back().regexgrp_ct, context_stack.back().regexgrps, tag);
 				}
 			}
 		}
@@ -345,14 +344,22 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 	else if (tag.type & T_NUMERICAL) {
 		for (const auto& mter : reading.tags_numerical) {
 			const Tag& itag = *(mter.second);
-			double compval = tag.comparison_val;
-			if (compval <= NUMERIC_MIN) {
-				compval = reading.parent->getMin(tag.comparison_hash);
-			}
-			else if (compval >= NUMERIC_MAX) {
-				compval = reading.parent->getMax(tag.comparison_hash);
-			}
 			if (tag.comparison_hash == itag.comparison_hash) {
+				double compval = tag.comparison_val;
+				if ((tag.type & T_NUMERIC_MATH) && tag.comparison_offset) {
+					MathParser mp(reading.parent->getMin(tag.comparison_hash), reading.parent->getMax(tag.comparison_hash));
+					UStringView exp(tag.tag);
+					exp.remove_prefix(tag.comparison_offset);
+					exp.remove_suffix(1);
+					compval = mp.eval(exp);
+				}
+				else if (compval <= NUMERIC_MIN) {
+					compval = reading.parent->getMin(tag.comparison_hash);
+				}
+				else if (compval >= NUMERIC_MAX) {
+					compval = reading.parent->getMax(tag.comparison_hash);
+				}
+
 				if (tag.comparison_op == OP_EQUALS && itag.comparison_op == OP_EQUALS && compval == itag.comparison_val) {
 					match = itag.hash;
 				}
@@ -467,28 +474,48 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 			}
 		}
 	}
-	else if (tag.type & T_VARIABLE) {
+	else if (tag.type & (T_VARIABLE|T_LOCAL_VARIABLE)) {
 		match = 0;
-		auto it = variables.find(tag.comparison_hash);
-		if (it != variables.end()) {
-			if (tag.variable_hash == 0) {
-				match = tag.hash;
+		auto& vars = [&]() -> auto& {
+			if (reading.parent->parent == gWindow->current || !(tag.type & T_LOCAL_VARIABLE)) {
+				return variables;
+			}
+			return reading.parent->parent->variables_set;
+		}();
+
+		auto kit = grammar->single_tags.find(tag.comparison_hash);
+		if (kit != grammar->single_tags.end()) {
+			auto key = kit->second;
+			auto it = vars.begin();
+			if (key->type & T_REGEXP) {
+				it = std::find_if(it, vars.end(), [&](auto& kv) { return doesTagMatchRegexp(kv.first, *key, bypass_index); });
+			}
+			else if (key->type & T_CASE_INSENSITIVE) {
+				it = std::find_if(it, vars.end(), [&](auto& kv) { return doesTagMatchIcase(kv.first, *key, bypass_index); });
 			}
 			else {
-				auto comp = grammar->single_tags.find(tag.variable_hash)->second;
-				if (comp->type & T_REGEXP) {
-					if (doesTagMatchRegexp(it->second, *comp, bypass_index)) {
-						match = tag.hash;
-					}
-				}
-				else if (comp->type & T_CASE_INSENSITIVE) {
-					if (doesTagMatchIcase(it->second, *comp, bypass_index)) {
-						match = tag.hash;
-					}
-				}
-				else if (comp->hash == it->second) {
+				it = vars.find(tag.comparison_hash);
+			}
+			if (it != vars.end()) {
+				if (tag.variable_hash == 0) {
 					match = tag.hash;
 				}
+				else {
+					auto comp = grammar->single_tags.find(tag.variable_hash)->second;
+					if (comp->type & T_REGEXP) {
+						if (doesTagMatchRegexp(it->second, *comp, bypass_index)) {
+							match = tag.hash;
+						}
+					}
+					else if (comp->type & T_CASE_INSENSITIVE) {
+						if (doesTagMatchIcase(it->second, *comp, bypass_index)) {
+							match = tag.hash;
+						}
+					}
+					else if (comp->hash == it->second) {
+						match = tag.hash;
+					}
+				}
 			}
 		}
 	}
@@ -503,22 +530,25 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		}
 	}
 	else if (tag.type & T_ENCL) {
-		if (!reading.parent->enclosed.empty()) {
+		auto sw = reading.parent->parent;
+		auto c = std::find(sw->all_cohorts.begin() + reading.parent->local_number, sw->all_cohorts.end(), reading.parent);
+		++c;
+		if (c != sw->all_cohorts.end() && (*c)->enclosed) {
 			match = true;
 		}
 	}
 	else if (tag.type & T_TARGET) {
-		if (target && reading.parent == target) {
+		if (rule_target && reading.parent == rule_target) {
 			match = grammar->tag_any;
 		}
 	}
 	else if (tag.type & T_MARK) {
-		if (mark && reading.parent == mark) {
+		if (reading.parent == get_mark()) {
 			match = grammar->tag_any;
 		}
 	}
 	else if (tag.type & T_ATTACHTO) {
-		if (attach_to && reading.parent == attach_to) {
+		if (reading.parent == get_attach_to().cohort) {
 			match = grammar->tag_any;
 		}
 	}
@@ -527,6 +557,16 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 			match = grammar->tag_any;
 		}
 	}
+	else if (tag.type & T_CONTEXT) {
+		if (context_stack.size() > 1) {
+			auto& list = context_stack[context_stack.size()-2].context;
+			if (tag.context_ref_pos <= list.size()) {
+				if (reading.parent == list[tag.context_ref_pos-1]) {
+					match = grammar->tag_any;
+				}
+			}
+		}
+	}
 
 	if (match) {
 		++match_single;
@@ -544,12 +584,8 @@ bool GrammarApplicator::doesSetMatchReading_trie(const Reading& reading, const S
 				continue;
 			}
 			if (kv.second.terminal) {
-				if (unif_mode) {
-					auto it = unif_tags->find(theset.number);
-					if (it != unif_tags->end() && it->second != &kv) {
-						continue;
-					}
-					(*unif_tags)[theset.number] = &kv;
+				if (unif_mode && !check_unif_tags(theset.number, &kv)) {
+					continue;
 				}
 				return true;
 			}
@@ -564,7 +600,7 @@ bool GrammarApplicator::doesSetMatchReading_trie(const Reading& reading, const S
 /**
  * Tests whether a given reading matches a given LIST set.
  *
- * In the https://visl.sdu.dk/cg3_performance.html test data, this function is executed 1073969 times.
+ * In the https://edu.visl.dk/cg3_performance.html test data, this function is executed 1073969 times.
  *
  * @param[in] reading The reading to test
  * @param[in] set The hash of the set to test against
@@ -589,13 +625,9 @@ bool GrammarApplicator::doesSetMatchReading_tags(const Reading& reading, const S
 		while (oiter != reading.tags_plain.end() && iiter != theset.trie.end()) {
 			if (*oiter == iiter->first->hash) {
 				if (iiter->second.terminal) {
-					if (unif_mode) {
-						auto it = unif_tags->find(theset.number);
-						if (it != unif_tags->end() && it->second != &*iiter) {
-							++iiter;
-							continue;
-						}
-						(*unif_tags)[theset.number] = &*iiter;
+					if (unif_mode && !check_unif_tags(theset.number, &*iiter)) {
+						++iiter;
+						continue;
 					}
 					retval = true;
 					break;
@@ -625,7 +657,7 @@ bool GrammarApplicator::doesSetMatchReading_tags(const Reading& reading, const S
 /**
  * Tests whether a given reading matches a given LIST or SET set.
  *
- * In the https://visl.sdu.dk/cg3_performance.html test data, this function is executed 5746792 times,
+ * In the https://edu.visl.dk/cg3_performance.html test data, this function is executed 5746792 times,
  * of which only 1700292 make it past the first index check.
  *
  * @param[in] reading The reading to test
@@ -639,21 +671,16 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 	// Only 30% of tests get past this.
 	// ToDo: This is not good enough...while numeric tags are special, their failures can be indexed.
 	if (!bypass_index && !unif_mode) {
-		if (index_readingSet_no[set].find(reading.hash) != index_readingSet_no[set].end()) {
+		if (index_readingSet_no[set].contains(reading.hash)) {
 			return false;
 		}
-		if (index_readingSet_yes[set].find(reading.hash) != index_readingSet_yes[set].end()) {
+		if (index_readingSet_yes[set].contains(reading.hash)) {
 			return true;
 		}
 	}
 
 	bool retval = false;
 
-	ticks tstamp(gtimer);
-	if (statistics) {
-		tstamp = getticks();
-	}
-
 	// ToDo: Make all places have Set* directly so we don't need to perform this lookup.
 	const Set& theset = *grammar->sets_list[set];
 
@@ -668,7 +695,7 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 	// &&unified sets
 	else if (theset.type & ST_SET_UNIFY) {
 		// First time, figure out all the sub-sets that match the reading and store them for later comparison
-		auto& usets = (*unif_sets)[theset.number];
+		auto& usets = (*context_stack.back().unif_sets)[theset.number];
 		if (usets.empty()) {
 			const Set& uset = *grammar->sets_list[theset.sets[0]];
 			const size_t size = uset.sets.size();
@@ -738,7 +765,8 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 			}
 		}
 		// Propagate unified tag to other sets of this set, if applicable
-		if (unif_mode || (theset.type & ST_TAG_UNIFY)) {
+		if ((unif_mode || (theset.type & ST_TAG_UNIFY)) && !context_stack.empty()) {
+			auto unif_tags = context_stack.back().unif_tags;
 			const void* tag = nullptr;
 			for (size_t i = 0; i < size; ++i) {
 				auto it = unif_tags->find(theset.sets[i]);
@@ -754,16 +782,6 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 			}
 		}
 	}
-	if (statistics) {
-		if (retval) {
-			theset.num_match++;
-		}
-		else {
-			theset.num_fail++;
-		}
-		ticks tmp = getticks();
-		theset.total_time += elapsed(tmp, tstamp);
-	}
 
 	// Store the result in the indexes in hopes that later runs can pull it directly from them.
 	if (retval) {
@@ -833,11 +851,11 @@ inline bool GrammarApplicator::doesSetMatchCohort_helper(Cohort& cohort, Reading
 	bool retval = false;
 	auto utags = ss_utags.get();
 	auto usets = ss_usets.get();
-	uint8_t orz = regexgrps.first;
+	uint8_t orz = (context_stack.empty() ? 0 : context_stack.back().regexgrp_ct);
 
-	if (context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY)) {
-		*utags = *unif_tags;
-		*usets = *unif_sets;
+	if (context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY) && !context_stack.empty()) {
+		*utags = *context_stack.back().unif_tags;
+		*usets = *context_stack.back().unif_sets;
 	}
 	if (doesSetMatchReading(reading, theset.number, (theset.type & (ST_CHILD_UNIFY | ST_SPECIAL)) != 0)) {
 		retval = true;
@@ -855,16 +873,22 @@ inline bool GrammarApplicator::doesSetMatchCohort_helper(Cohort& cohort, Reading
 		retval = doesSetMatchCohort_testLinked(cohort, theset, context);
 		if (context->options & POS_ATTACH_TO) {
 			reading.matched_tests = retval;
+			if (retval && !context_stack.empty()) {
+				context_stack.back().attach_to.cohort = &cohort;
+				// This will be set by doesSetMatchCohortNormal
+				context_stack.back().attach_to.reading = nullptr;
+				context_stack.back().attach_to.subreading = &reading;
+			}
 		}
 	}
-	if (!retval && context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY) && (utags->size() != unif_tags->size() || *utags != *unif_tags)) {
-		unif_tags->swap(utags);
+	if (!retval && context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY) && !context_stack.empty() && (utags->size() != context_stack.back().unif_tags->size() || *utags != *context_stack.back().unif_tags)) {
+		context_stack.back().unif_tags->swap(utags);
 	}
-	if (!retval && context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY) && usets->size() != unif_sets->size()) {
-		unif_sets->swap(usets);
+	if (!retval && context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY) && !context_stack.empty() && usets->size() != context_stack.back().unif_sets->size()) {
+		context_stack.back().unif_sets->swap(usets);
 	}
-	if (!retval) {
-		regexgrps.first = orz;
+	if (!retval && !context_stack.empty()) {
+		context_stack.back().regexgrp_ct = orz;
 	}
 	return retval;
 }
@@ -902,6 +926,7 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 			continue;
 		}
 		for (auto reading : *list) {
+			Reading* reading_head = reading;
 			if (context && context->test) {
 				// ToDo: Barriers need some way to escape sub-readings
 				reading = get_sub_reading(reading, context->test->offset_sub);
@@ -917,6 +942,13 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 			}
 			if (doesSetMatchCohort_helper(cohort, *reading, *theset, context)) {
 				retval = true;
+				// if there's a subreading, _helper doesn't know the
+				// parent, so set it here
+				if (!context_stack.empty() &&
+					context_stack.back().attach_to.cohort == &cohort &&
+					context_stack.back().attach_to.subreading == reading) {
+					context_stack.back().attach_to.reading = reading_head;
+				}
 			}
 			if (retval && (!context || !(context->test && context->test->linked) || context->did_test)) {
 				return retval;
