@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -67,6 +65,9 @@ enum RULE_FLAGS : uint64_t {
 	RF_AFTER        = (1 << 28),
 	RF_IGNORED      = (1 << 29),
 	RF_LOOKIGNORED  = (1 << 30),
+	RF_NOMAPPED     = (1ull << 31),
+	RF_NOPARENT     = (1ull << 32),
+	RF_DETACH       = (1ull << 33),
 };
 
 using rule_flags_t = std::underlying_type<RULE_FLAGS>::type;
@@ -94,6 +95,9 @@ constexpr auto init_flag_excls(rule_flags_t v) {
 
 constexpr auto _flags_excls = make_array<FLAGS_COUNT>(init_flag_excls);
 
+class Rule;
+typedef std::vector<Rule*> RuleVector;
+
 class Rule {
 public:
 	UString name;
@@ -102,32 +106,26 @@ public:
 	uint32_t childset1 = 0, childset2 = 0;
 	uint32_t line = 0, number = 0;
 	uint32_t varname = 0, varvalue = 0; // ToDo: varvalue is unused
-	uint32_t flags = 0;
+	uint64_t flags = 0;
 	int32_t section = 0;
 	int32_t sub_reading = 0;
 	KEYWORDS type = K_IGNORE;
 	Set* maplist = nullptr;
 	Set* sublist = nullptr;
+	RuleVector sub_rules;
 
 	mutable ContextList tests;
 	mutable ContextList dep_tests;
-	mutable uint32_t num_fail = 0, num_match = 0;
-	mutable double total_time = 0;
 	mutable ContextualTest* dep_target = nullptr;
 
 	Rule() = default;
 	~Rule() = default;
 	void setName(const UChar* to);
 
-	void resetStatistics();
-
 	void addContextualTest(ContextualTest* to, ContextList& head);
 	void reverseContextualTests();
-
-	static bool cmp_quality(const Rule* a, const Rule* b);
 };
 
-typedef std::vector<Rule*> RuleVector;
 typedef std::map<uint32_t, Rule*> RuleByLineMap;
 typedef std::unordered_map<uint32_t, Rule*> RuleByLineHashMap;
 }
