@@ -1,28 +1,31 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
 #ifndef c6d28b7452ec699b_INLINES_H
 #define c6d28b7452ec699b_INLINES_H
 
+template<typename T>
+constexpr inline int8_t SI8(T t) {
+	return static_cast<int8_t>(t);
+}
+
 template<typename T>
 constexpr inline int32_t SI32(T t) {
 	return static_cast<int32_t>(t);
@@ -53,10 +56,25 @@ constexpr inline uint64_t UI64(T t) {
 	return static_cast<uint64_t>(t);
 }
 
+template<typename T>
+constexpr inline double DBL(T t) {
+	return static_cast<double>(t);
+}
+
+template<typename T>
+constexpr inline size_t UIZ(T t) {
+	return static_cast<size_t>(t);
+}
+
+template<typename T>
+constexpr inline void* VOIDP(T t) {
+	return static_cast<void*>(t);
+}
+
 namespace CG3 {
 
-constexpr double NUMERIC_MIN = static_cast<double>(-(1ll << 48ll));
-constexpr double NUMERIC_MAX = static_cast<double>((1ll << 48ll) - 1);
+constexpr double NUMERIC_MIN = DBL(-(1ll << 48ll));
+constexpr double NUMERIC_MAX = DBL((1ll << 48ll) - 1);
 constexpr uint32_t CG3_HASH_SEED = 705577479u;
 
 /*
@@ -247,6 +265,14 @@ inline UStringView USV(UnicodeString& str) {
 	return UStringView(str.getTerminatedBuffer(), str.length());
 }
 
+inline UStringView USV(UString& str) {
+	return UStringView(str);
+}
+
+inline bool ISDELIM(const UChar c) {
+	return c == '(' || c == ')' || c == '+' || c == '-' || c == '*' || c == '/' || c == '^' || c == '%' || c == '=';
+}
+
 inline bool ISSPACE(const UChar c) {
 	if (c <= 0xFF && c != 0x09 && c != 0x0A && c != 0x0D && c != 0x20 && c != 0xA0) {
 		return false;
@@ -256,10 +282,10 @@ inline bool ISSPACE(const UChar c) {
 
 template<typename Char>
 inline bool ISSTRING(const Char* p, const uint32_t c) {
-	if (*(p - 1) == '"' && *(p + c + 1) == '"') {
+	if (p[-1] == '"' && p[c + 1] == '"') {
 		return true;
 	}
-	if (*(p - 1) == '<' && *(p + c + 1) == '>') {
+	if (p[-1] == '<' && p[c + 1] == '>') {
 		return true;
 	}
 	return false;
@@ -289,18 +315,32 @@ inline bool ISSPACE(const Char* p) {
 	return ISSPACE(*p) && !ISESC(p);
 }
 
+template<typename Char>
+inline bool ISALPHA_C(Char p) {
+	return (p < 255) && isalpha(p);
+}
+
+template<typename Char>
+inline bool ISDIGIT_C(Char p) {
+	return (p < 255) && isdigit(p);
+}
+
 template<typename Char, typename C, size_t N>
-inline bool IS_ICASE(const Char* p, const C (&uc)[N], const C (&lc)[N]) {
+inline size_t IS_ICASE(const Char* p, const C (&uc)[N], const C (&lc)[N]) {
 	// N - 1 due to null terminator for string constants
 	if (ISSTRING(p, N - 1)) {
-		return false;
+		return 0;
 	}
-	for (size_t i = 0; i < N - 1; ++i) {
+	size_t i = 0;
+	for (; i < N - 1; ++i) {
 		if (p[i] != uc[i] && p[i] != lc[i]) {
-			return false;
+			return 0;
 		}
 	}
-	return true;
+	if (!u_isalnum(p[N - 1])) {
+		return i;
+	}
+	return 0;
 }
 
 template<typename Char>
@@ -435,11 +475,6 @@ inline bool is_cg3b(const S& s) {
 	return (s[0] == 'C' && s[1] == 'G' && s[2] == '3' && s[3] == 'B');
 }
 
-template<typename Cont, typename VT>
-inline bool index_matches(const Cont& index, const VT& entry) {
-	return (index.find(entry) != index.end());
-}
-
 inline void insert_if_exists(boost::dynamic_bitset<>& cont, const boost::dynamic_bitset<>* other) {
 	if (other && !other->empty()) {
 		cont.resize(std::max(cont.size(), other->size()));
@@ -499,6 +534,43 @@ inline UString readUTF8String(S& input) {
 	#pragma warning (disable: 4127)
 #endif
 
+// macOS has macros for these that somehow only reveal themselves in the Python binding
+#ifdef htonl
+#undef htonl
+#endif
+
+#ifdef htons
+#undef htons
+#endif
+
+#ifdef ntohl
+#undef ntohl
+#endif
+
+#ifdef ntohs
+#undef ntohs
+#endif
+
+template<typename T>
+T htonl(T t) {
+	return be::native_to_big(t);
+}
+
+template<typename T>
+T htons(T t) {
+	return be::native_to_big(t);
+}
+
+template<typename T>
+T ntohl(T t) {
+	return be::big_to_native(t);
+}
+
+template<typename T>
+T ntohs(T t) {
+	return be::big_to_native(t);
+}
+
 inline uint32_t hton32(uint32_t val) {
 	return UI32(htonl(val));
 }
@@ -540,7 +612,7 @@ inline void writeSwapped(std::ostream& stream, const T& value) {
 template<>
 inline void writeSwapped(std::ostream& stream, const double& value) {
 	int exp = 0;
-	auto mant64 = UI64(SI64(std::numeric_limits<int64_t>::max() * frexp(value, &exp)));
+	auto mant64 = UI64(SI64(DBL(std::numeric_limits<int64_t>::max()) * frexp(value, &exp)));
 	auto exp32 = UI32(exp);
 	writeSwapped(stream, mant64);
 	writeSwapped(stream, exp32);
@@ -584,7 +656,7 @@ inline double readSwapped(std::istream& stream) {
 	auto mant64 = readSwapped<uint64_t>(stream);
 	auto exp = static_cast<int>(readSwapped<int32_t>(stream));
 
-	auto value = static_cast<double>(SI64(mant64)) / std::numeric_limits<int64_t>::max();
+	auto value = DBL(SI64(mant64)) / DBL(std::numeric_limits<int64_t>::max());
 
 	return ldexp(value, exp);
 }
@@ -700,6 +772,27 @@ private:
 	T* p;
 };
 
+class scope_guard {
+public:
+	scope_guard(std::function<void()> func, bool good = true)
+		: func(func)
+		, good(good)
+	{}
+
+	~scope_guard() {
+		if (good) {
+			func();
+		}
+	}
+
+	void set(bool val = true) {
+		good = val;
+	}
+private:
+	std::function<void()> func;
+	bool good = true;
+};
+
 template<typename T>
 inline T* reverse(T* head) {
 	T* nr = nullptr;
@@ -712,6 +805,26 @@ inline T* reverse(T* head) {
 	return nr;
 }
 
+template <typename T>
+struct Reversed {
+	T& t;
+};
+
+template <typename T>
+auto begin(Reversed<T> c) {
+	return std::rbegin(c.t);
+}
+
+template <typename T>
+auto end(Reversed<T> c) {
+	return std::rend(c.t);
+}
+
+template <typename T>
+Reversed<T> reversed(T&& c) {
+	return { c };
+}
+
 template<typename Cont, typename T>
 inline void erase(Cont& cont, const T& val) {
 	cont.erase(std::remove(cont.begin(), cont.end(), val), cont.end());
@@ -740,62 +853,6 @@ inline size_t fwrite_throw(const void* buffer, size_t size, size_t count, FILE*
 	return rv;
 }
 
-template<typename Pool, typename Var>
-void pool_get(Pool& pool, Var& var) {
-	if (!pool.empty()) {
-		var.swap(pool.back());
-		var.clear();
-		pool.pop_back();
-	}
-}
-
-template<typename Pool, typename Var>
-void pool_put(Pool& pool, Var& var) {
-	pool.resize(pool.size() + 1);
-	var.swap(pool.back());
-}
-
-template<typename Pool, typename Var>
-void pool_get(Pool& pool, Var*& var) {
-	var = nullptr;
-	if (!pool.empty()) {
-		var = pool.back();
-		pool.pop_back();
-	}
-}
-
-template<typename Pool>
-typename Pool::value_type pool_get(Pool& pool) {
-	typename Pool::value_type var = nullptr;
-	if (!pool.empty()) {
-		var = pool.back();
-		pool.pop_back();
-	}
-	return var;
-}
-
-template<typename Pool, typename Var>
-void pool_put(Pool& pool, Var* var) {
-	var->clear();
-	pool.push_back(var);
-}
-
-template<typename Pool>
-struct pool_cleaner {
-	Pool& pool;
-
-	pool_cleaner(Pool& pool)
-	  : pool(pool)
-	{
-	}
-
-	~pool_cleaner() {
-		for (auto p : pool) {
-			delete p;
-		}
-	}
-};
-
 template<class Function, std::size_t... Indices>
 constexpr auto make_array_helper(Function f, std::index_sequence<Indices...>) -> std::array<typename std::invoke_result<Function, std::size_t>::type, sizeof...(Indices)> {
 	return { { f(Indices)... } };
@@ -806,6 +863,24 @@ constexpr auto make_array(Function f) -> std::array<typename std::invoke_result<
 	return make_array_helper(f, std::make_index_sequence<N>{});
 }
 
+namespace details {
+	inline void _concat(std::string&) {
+	}
+
+	template<typename T, typename... Args>
+	inline void _concat(std::string& msg, const T& t, Args... args) {
+		msg.append(t);
+		_concat(msg, args...);
+	}
+}
+
+template<typename T, typename... Args>
+inline std::string concat(const T& value, Args... args) {
+	std::string msg(value);
+	details::_concat(msg, args...);
+	return msg;
+}
+
 }
 
 #endif
