@@ -1,22 +1,20 @@
 /*
-* Copyright (C) 2007-2021, GrammarSoft ApS
+* Copyright (C) 2007-2025, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
-* This file is part of VISL CG-3
-*
-* VISL CG-3 is free software: you can redistribute it and/or modify
+* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
-* VISL CG-3 is distributed in the hope that it will be useful,
+* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with VISL CG-3.  If not, see <http://www.gnu.org/licenses/>.
+* along with this progam.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #pragma once
@@ -31,7 +29,7 @@
 namespace CG3 {
 
 template<typename State>
-Tag* parseTag(const UChar* to, const UChar* p, State& state) {
+Tag* parseTag(const UChar* to, const UChar* p, State& state, bool unescape=true) {
 	if (to[0] == 0) {
 		state.error("%s: Error: Empty tag on line %u near `%S`! Forgot to fill in a ()?\n", p);
 	}
@@ -78,6 +76,11 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 			tmp += 4;
 			length -= 4;
 		}
+		if (tmp[0] == 'L' && tmp[1] == 'V' && tmp[2] == 'A' && tmp[3] == 'R' && tmp[4] == ':') {
+			tag->type |= T_LOCAL_VARIABLE;
+			tmp += 5;
+			length -= 5;
+		}
 		if (tmp[0] == 'S' && tmp[1] == 'E' && tmp[2] == 'T' && tmp[3] == ':') {
 			tag->type |= T_SET;
 			tmp += 4;
@@ -96,7 +99,7 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 			goto label_isVarstring;
 		}
 
-		if (tmp[0] && (tmp[0] == '"' || tmp[0] == '<' || tmp[0] == '/')) {
+		if (tmp[0] && !(tag->type & (T_VARIABLE|T_LOCAL_VARIABLE)) && (tmp[0] == '"' || tmp[0] == '<' || tmp[0] == '/')) {
 			size_t oldlength = length;
 
 			// Parse the suffixes r, i, v but max only one of each.
@@ -149,7 +152,7 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 		}
 
 		for (size_t i = 0, oldlength = length; tmp[i] != 0 && i < oldlength; ++i) {
-			if (tmp[i] == '\\') {
+			if (unescape && tmp[i] == '\\') {
 				++i;
 				--length;
 			}
@@ -190,18 +193,25 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 			}
 		}
 
-		if ((tag->type & T_VARIABLE) && tag->tag.find('=') != UString::npos) {
+		if (tag->type & (T_VARIABLE|T_LOCAL_VARIABLE)) {
 			size_t pos = tag->tag.find('=');
-			tag->comparison_op = OP_EQUALS;
-			tag->variable_hash = parseTag(&tag->tag[pos + 1], p, state)->hash;
-			tag->comparison_hash = hash_value(tag->tag.substr(0, pos));
+			if (pos != UString::npos) {
+				tag->comparison_op = OP_EQUALS;
+				tag->variable_hash = parseTag(&tag->tag[pos + 1], p, state, false)->hash;
+				tag->tag[pos] = 0;
+				tag->comparison_hash = parseTag(tag->tag.c_str(), p, state, false)->hash;
+				tag->tag[pos] = '=';
+			}
+			else {
+				tag->comparison_hash = parseTag(tag->tag.c_str(), p, state, false)->hash;
+			}
 		}
 		else {
 			tag->comparison_hash = hash_value(tag->tag);
 		}
 
-		if (tag->tag[0] == '<' && tag->tag[length - 1] == '>') {
-			tag->parseNumeric();
+		if (tag->tag[0] == '<' && tag->tag[length - 1] == '>' && !(tag->type & (T_CASE_INSENSITIVE | T_REGEXP | T_REGEXP_LINE | T_VARSTRING))) {
+			tag->parseNumeric(true);
 		}
 		/*
 		if (tag->tag[0] == '#') {
@@ -241,6 +251,42 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 		else if (tag->tag == STR_UU_SAME_BASIC) {
 			tag->type |= T_SAME_BASIC;
 		}
+		else if (tag->tag == STR_UU_C1) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 1;
+		}
+		else if (tag->tag == STR_UU_C2) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 2;
+		}
+		else if (tag->tag == STR_UU_C3) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 3;
+		}
+		else if (tag->tag == STR_UU_C4) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 4;
+		}
+		else if (tag->tag == STR_UU_C5) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 5;
+		}
+		else if (tag->tag == STR_UU_C6) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 6;
+		}
+		else if (tag->tag == STR_UU_C7) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 7;
+		}
+		else if (tag->tag == STR_UU_C8) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 8;
+		}
+		else if (tag->tag == STR_UU_C9) {
+			tag->type |= T_CONTEXT;
+			tag->context_ref_pos = 9;
+		}
 
 		if (tag->type & T_REGEXP) {
 			if (tag->tag == STR_RXTEXT_ANY || tag->tag == STR_RXBASE_ANY || tag->tag == STR_RXWORD_ANY) {
@@ -288,10 +334,14 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 		tag->type |= T_SPECIAL;
 	}
 
-	if (tag->type & T_VARSTRING && tag->type & (T_REGEXP | T_REGEXP_ANY | T_VARIABLE | T_META)) {
+	if (tag->type & T_VARSTRING && tag->type & (T_REGEXP | T_REGEXP_ANY | T_VARIABLE | T_LOCAL_VARIABLE | T_META)) {
 		state.error("%s: Error: Tag %S cannot mix varstring with any other special feature on line %u near `%S`!\n", to, p);
 	}
 
+	if (USV(tag->tag) != to) {
+		tag->tag_raw = to;
+	}
+
 	return state.addTag(tag);
 }
 
